DEFAULT_INDENT = 4
SORT_KEYS_BY_DEFAULT = True
REMOVE_EMPTY_VALUES_BY_DEFAULT = False

#: Collection types.
HOSTS = 'hosts'
ASSETS = 'assets'
VULNERABILITIES = 'vulnerabilities'
COLLECTION_TYPES = [HOSTS, ASSETS, VULNERABILITIES]
